#!/usr/bin/perl
#
# Module for parsing secrets files,
# such as chap-secrets or pap-secrets.
#
# v0.11
#

package Edit::Secrets;

use strict;
use base qw(Exporter);

sub new
{
    my $class = shift;
    my $self = { 'file' => shift };
    
    bless $self, $class;
    
    return $self;
}

sub read
{
    my $self = shift;
    %{$self->{'data'}} = ();
    
    open (SECRETS, "<$self->{'file'}") || return;
	my @data = <SECRETS>;
	close (SECRETS);
    
    foreach my $str (@data)
    {
	$str =~ s/\#.*$//g;
	$str =~ s/(^\s+)|(\s+$)//g;
	next if ($str eq "");
	
	# Split string of settings.
	# 0 - unit name, 1 - server name, 2 - password, 3 - IP-address
	my @str = split (/\s+/, $str);
	
	map ($_ =~ s/"//g, @str);
	
	if (defined($self->{'data'}{$str[0]}))
	{
	    warn ("Duplicate unit name \"$str[0]\" on read.");
	    return;
	}
	$self->{'data'}{$str[0]} = [ $str[1], $str[2], $str[3] ];
    }

    return 1;
}

sub edit
{
    my $self = shift;
    
    # $new == 0 - record exists
    # $new == 1 - record not exists
    my ($new, $name, $server, $password, $ip) = @_;

    if ($new && defined($self->{'data'}{$name}))
    {
	warn ("Duplicate unit name \"$name\" on edit.");
	return;
    }
		
    $self->{'data'}{$name} = [ $server, $password, $ip ];

    return 1;
}

sub del
{
    my $self = shift;
    my $target = shift;
    
    delete $self->{'data'}{$target} || return;
    
    return 1;
}

sub write
{
    my $self = shift;
    
    open (SECRETS, ">$self->{'file'}") || return;
    foreach my $key (keys %{$self->{'data'}})
    {
	printf SECRETS ("%-22s%-15s%-22s%s\n", "\"$key\"", $self->{'data'}{$key}[0], "\"$self->{'data'}{$key}[1]\"", $self->{'data'}{$key}[2]);
    }
    close (SECRETS);

    return 1;
}

1;
