use strict;

###########################
sub policyLoadData {
###########################
    my (%POLICY, %POLICY_BYNAME);
    
    my $result = netams_cmdln("show policy");
    $result =~ s/target://g;
	
    my @arr1 = split(/\n/, $result);
    shift @arr1;
    foreach (@arr1) 
    {
	my @arr2 = split(/\|/, $_);
	
	foreach (@arr2) { s/(^\s+)|(\s+$)//g }
	
	$POLICY{$arr2[1]}{oid}	= $arr2[1];
	$POLICY{$arr2[1]}{type}	= $arr2[0];
	$POLICY{$arr2[1]}{name}	= $arr2[2];
	
	if ($arr2[3] =~ /tcp-ports/)
	{
	    $POLICY{$arr2[1]}{targ}	= "tcp-ports";   
	    ($POLICY{$arr2[1]}{ports})	= $arr2[3] =~ /^tcp-ports (.*)$/;
	}
	else
	{
	    $POLICY{$arr2[1]}{targ}	= $arr2[3];
	    $POLICY{$arr2[1]}{ports}	= "";
	}
	
	$POLICY_BYNAME{$arr2[2]} = $POLICY{$arr2[1]};
    }
    
    return (\%POLICY, \%POLICY_BYNAME);
}

########################
sub quotaLoadData {
########################
    my (%QUOTA, %QUOTA_INFO);
    
    netams_send("show quota");
	
    my $result = netams_readline();
    $result =~ s/quota//g;
    $result =~ s/in:/in/g;
    $result =~ s/out:/out/g;
    $result =~ s/sum:/sum/g;
    $result =~ s/\n/<br>/g;
    $result =~ s/\s/,/g;
    $result =~ s/,+/,/g;
    $result =~ s/<br>,/<br>/g;

    my $currunit;
    
    my @res = split("<br>", $result);
    foreach (@res) 
    {
	my @str = split(",", $_);
	
	next unless (defined $str[3]);
	
	if ($str[0] eq "OID:") 
	{
	    $currunit = $str[1];

	    $QUOTA{$currunit}{policy}	= $str[4];
	    $QUOTA{$currunit}{treshold}	= $str[6];
	    $QUOTA{$currunit}{quota}	= $str[7];
	    $QUOTA{$currunit}{notify}	= $str[8];
	    $QUOTA{$currunit}{status}	= $str[9] || "ENABLED";
	}
	elsif (	$str[0] eq "DAY" || 
		$str[0] eq "MONTH" || 
		$str[0] eq "TOTAL" || 
		$str[0] eq "HOUR") 
	{
	    # [0] - period, [1] - destination, [3] - soft or hard.
	    $QUOTA{$currunit}{$str[0]}{$str[1]}{$str[3]}{used}	    = $str[2];
	    $QUOTA{$currunit}{$str[0]}{$str[1]}{$str[3]}{limit}	    = $str[4];
	    $QUOTA{$currunit}{$str[0]}{$str[1]}{$str[3]}{ratio}	    = $str[6];
	    $QUOTA{$currunit}{$str[0]}{$str[1]}{$str[3]}{status}    = $str[8];
	}
    }
	
    my @str = split(",", pop @res);

    $QUOTA_INFO{units}	    = $str[2];
    $QUOTA_INFO{enabled}    = $str[4];
    $QUOTA_INFO{active}	    = $str[6];
    $QUOTA_INFO{blocked}    = $str[9];
    $QUOTA_INFO{notified}   = $str[9];

    return (\%QUOTA, \%QUOTA_INFO);	
}

########################
sub unitLoadData {
########################
    my (%UNIT, %UNIT_BYNAME);
   
    # Create unit list
    my $result	= netams_cmdln("show units");
    $result =~ s/(Sub: )|(IP: )|(IPs: )//g;
    
    my @arr1 = split(/\n/, $result);
    shift @arr1;
    foreach (@arr1) 
    {
	# Remove spaces at a start and an end of string,
	# cannot be done when splitting
	s/(^\s+)|(\s+$)//g;
	# Split string
	my @arr2 = split(/\s*\|\s*/, $_);
	
	$UNIT{$arr2[1]}{type}	= $arr2[0];
	$UNIT{$arr2[1]}{name}	= $arr2[2];
	$UNIT{$arr2[1]}{oid}	= $arr2[1];
	$UNIT{$arr2[1]}{parent}	= $arr2[4];
	$UNIT{$arr2[1]}{nlp}	= $arr2[3];
	$UNIT{$arr2[1]}{email}	= $arr2[5];
	$UNIT{$arr2[1]}{params}	= $arr2[6];
		
	$UNIT_BYNAME{$arr2[2]}	= $UNIT{$arr2[1]};	
	#$UNIT_BYIP{$arr2[6]}	= $UNIT{$arr2[1]};
    }
   
    # For each unit, create list of used policies
    $result = netams_cmdln("show list");
    # Next line is needed, because there is two spaces after policy flag
    $result =~ s/([%!]) /$1,  /g;
    $result =~ s/  +//g;
    $result =~ s/ /,/g;
    $result =~ s/\n,/\n/g;
    
    my ($uid, $type);
    foreach (split(/\n/, $result)) 
    {
	my @res1 = split(",", $_);

	if	($res1[0] eq "OID:") { $uid = $res1[1] }
	elsif	($res1[0] eq "SYST" || 
		 $res1[0] eq "FW" || 
		 $res1[0] eq "ACCT") { $type = $res1[0] }
	elsif	($res1[0] ne "" && $res1[1] ne "") 
	{
	    next unless (exists $UNIT{$uid});
	    
	    if ($res1[0] eq "!" || $res1[0] eq "%" || $res1[0] eq "%!") 
	    {
		# $res1[0] - policy flag, $res1[1] - policy oid
		$UNIT{$uid}{policy}{$res1[1]}	= $res1[0];
		$UNIT{$uid}{policy}{list}	.= "$res1[1],";
	    }
	    else 
	    { 
		# $res1[0] - policy oid
		$UNIT{$uid}{policy}{$res1[0]}	= "";
		$UNIT{$uid}{policy}{list}	.= "$res1[0],";
	    }
	}
    }

    return (\%UNIT, \%UNIT_BYNAME);
}

###########################
sub loginLoadData {
###########################
    my %LOGIN;
    
    my $result=netams_cmdln("show login");
    $result=~ s/ +/,/g;
    $result=~ s/>//g;
    $result=~ s/\[/,/g;
    $result=~ s/\]//g;
    $result=~ s/OP:/OP:,/g;
    $result=~ s/LU:/LU:,/g;
    $result=~ s/inact:/inact:,/g;
    $result=~ s/abs:/abs:,/g;
    $result=~ s/,,+/,/g;
    $result=~ s/\n,/\n/g;
    
    my $currunit;

    my @res=split("\n", $result);
    foreach (@res) 
    {
	my @str = split(",", $_);
	if ($str[0] eq "OID:") 
	{ 
	    $currunit = $str[1]; 
	    $LOGIN{$currunit}{status}	= $str[3];
	    $LOGIN{$currunit}{inact}	= $str[5];
	    $LOGIN{$currunit}{'abs'}	= $str[7];
	}
	elsif ($str[0] eq "OP:") 
	{ 
	    $LOGIN{$currunit}{date1}	= $str[1];
	    $LOGIN{$currunit}{time1}	= $str[2];
	    $LOGIN{$currunit}{date2}	= $str[4];
	    $LOGIN{$currunit}{time2}	= $str[5];
	    $LOGIN{$currunit}{ip}	= $str[6];
	    $LOGIN{$currunit}{mac}	= $str[7];
	}
    }

    return \%LOGIN;
}

###########################
sub countLoadData {
###########################
    my %COUNT;
    
    my $result = netams_cmdln("show list full");	
    
    # Next line is needed, because there is two spaces after policy flag
    $result =~ s/([%!]) /$1,  /g;
    $result =~ s/  +//g;
    $result =~ s/in:|out://g;
    $result =~ s/\n/<br>/g;
    $result =~ s/\s/,/g;
    $result =~ s/,,/,/g;
    $result =~ s/<br>,/<br>/g;
    
    my ($currunit, $currpid);
    
    my @res = split("<br>", $result);
    foreach (@res) 
    {
	my @str = split(",", $_);
	
	next unless (defined $str[1]);
	
	if ($str[0] eq "OID:")		{ $currunit = $str[1] }
	# if $str[0] is six characters long, it is policy oid
	if ($str[0] =~ /\w\w\w\w\w\w/)	{ $currpid  = $str[0] }
	# if $str[0] contains % or !, it is flag, and $str[1] is policy oid
	if ($str[0] =~ /[%!]/)		{ $currpid  = $str[1] }
	if ($str[0] =~ /\d\d.\d\d.\d\d\d\d/) 
	{
	    $COUNT{$currunit}{$currpid}{$str[2]}{'date'}    = $str[0];
	    $COUNT{$currunit}{$currpid}{$str[2]}{'time'}    = $str[1];
	    $COUNT{$currunit}{$currpid}{$str[2]}{'in'}	    = $str[3];
	    $COUNT{$currunit}{$currpid}{$str[2]}{'out'}	    = $str[4];
	}
    }
    
    return \%COUNT;
}

1;
