## NeTAMS (www.netams.com) admintool by RamireZ (ramirez@ru.ru)
##
## TRAFFIC COUNTERS MODULE

use strict;

# Load data
my ($UNIT, $UNIT_BYNAME)	= &unitLoadData;
my ($POLICY, $POLICY_BYNAME)	= &policyLoadData;
my $COUNT			= &countLoadData;

# Months names, needed for detailed statistic
my @months = qw(           );

my %switch =
(
    default	=> \&showCount,
    months	=> \&detailedMonths,
    days	=> \&detailedDays,
    hours	=> \&detailedHours,
);

&{ &chooseAction(\%switch) };

########################
sub showCount {
######################## 

    # Make list of all units, for use in <option> tags.
    # If we are viewing traffic for specific unit (not all) 
    # mark it with "selected".
    my @units_cbo; 
    foreach my $uid (keys %$UNIT)
    {
	if ($query{uid} eq $uid) 
	{
	    push (@units_cbo, 
		{
		    uid => $uid, 
		    name => $UNIT->{$uid}{name}, 
		    selected => ' selected',
		}); 
	}
	else
	{
	    push (@units_cbo,
		{ 
		    uid => $uid, 
		    name => $UNIT->{$uid}{name},
		});
	}
    }

    # Create traffic list.
    # For all units if no uid defined, or for specific unit with given uid.
    my @stats;
    unless ($query{uid})
    {
	foreach my $uid (keys %$UNIT) { &_statsForUnit($uid, \@stats) }
    }
    else 
    { 
	&_statsForUnit($query{uid}, \@stats);
    }
    
    $tp = HTML::Template->new (filename => './html/count/showCount.tmpl');
    $tp->param(	
		    units_cbo   => \@units_cbo,
		    stats	    => \@stats,
		);
}

################################
sub _statsForUnit {
################################
    # UID for which to collect stats data
    my $uid = shift;
    # Reference to stats array
    my $stats = shift;

    my @stats_data;
    foreach my $pol (split(",", $UNIT->{$uid}{policy}{list})) 
    {
	if ($POLICY->{$pol}{type} eq "acct")
	{
	    push (@stats_data,
	    {
		fn	=> $query{fn},
		uid	=> $uid,
		pid	=> $pol,
		policy	=> $POLICY->{$pol}{name},
		flag	=> $UNIT->{$uid}{policy}{$pol},
		in_h	=> sprintf("%.3f", 
			    &convertToMB($COUNT->{$uid}{$pol}{hour}{in})),
		out_h	=> sprintf("%.3f",
			    &convertToMB($COUNT->{$uid}{$pol}{hour}{out})),
		in_d	=> sprintf("%.3f",
			    &convertToMB($COUNT->{$uid}{$pol}{day}{in})),
		out_d	=> sprintf("%.3f",
			    &convertToMB($COUNT->{$uid}{$pol}{day}{out})),
		in_m	=> sprintf("%.3f",
			    &convertToMB($COUNT->{$uid}{$pol}{month}{in})),
		out_m	=> sprintf("%.3f",
			    &convertToMB($COUNT->{$uid}{$pol}{month}{out})),
		in_t	=> sprintf("%.3f",
			    &convertToMB($COUNT->{$uid}{$pol}{total}{in})),
		out_t	=> sprintf("%.3f",
			    &convertToMB($COUNT->{$uid}{$pol}{total}{out})),
		detailed_stat => $cfg{detailed_stat},	
	    });
	}
    }
    
    push (@$stats, 
    { 
	name	    => $UNIT->{$uid}{name},
	uid	    => $uid,
	stats_data  => \@stats_data,
    });
}


sub detailedMonths
{
    my $dbh = &dbConnect;
    my @stats_by_months;

    my $sth = $dbh->prepare("
	SELECT bytes_in, bytes_out, FROM_UNIXTIME(t_from, '%Y%m')
	FROM summary
	WHERE prefix='M'
	    AND unit_oid=".$dbh->quote(hex($query{uid}))."
	    AND policy_oid=".$dbh->quote(hex($query{pid}))."
	ORDER by t_from desc");

    $sth->execute;
    
    while (my $data_ref = $sth->fetchrow_arrayref)
    {
	my ($year, $month) = unpack('A4 A2', $$data_ref[2]);
    
	push(@stats_by_months,
	{
	    fn	    => $query{fn},
	    uid	    => $query{uid},
	    pid	    => $query{pid},
	    mb_in   => sprintf("%.3f", &convertToMB($$data_ref[0])),
	    mb_out  => sprintf("%.3f", &convertToMB($$data_ref[1])),
	    year    => $year,
	    month   => $month,
	    month_word => $months[$month-1],
	});
    }

    $sth->finish;
    
    $tp = HTML::Template->new (filename => './html/count/detailedMonths.tmpl');
    $tp->param(	
		    uid		=> $query{uid},
		    name	=> $UNIT->{$query{uid}}{name},
		    pid		=> $query{pid},
		    policy	=> $POLICY->{$query{pid}}{name},
		    flag	=> $UNIT->{$query{uid}}{policy}{$query{pid}},
		    stats_by_months  => \@stats_by_months,
		);
}

sub detailedDays
{
    my $dbh = &dbConnect;
    my @stats_by_days;

    my $sth = $dbh->prepare("
	SELECT bytes_in, bytes_out, FROM_UNIXTIME(t_from, '%Y%m%d')
	FROM summary
	WHERE prefix='D'
	    AND unit_oid=".$dbh->quote(hex($query{uid}))."
	    AND policy_oid=".$dbh->quote(hex($query{pid}))."
	    AND FROM_UNIXTIME(t_from, '%Y%m')=".$dbh->quote($query{period})."
	ORDER by t_from desc");

    $sth->execute;
   
    my ($year, $month);
    while (my $data_ref = $sth->fetchrow_arrayref)
    {
	my $day;
	($year, $month, $day) = unpack('A4 A2 A2', $$data_ref[2]);
    
	push(@stats_by_days,
	{
	    fn	    => $query{fn},
	    uid	    => $query{uid},
	    pid	    => $query{pid},
	    mb_in   => sprintf("%.3f", &convertToMB($$data_ref[0])),
	    mb_out  => sprintf("%.3f", &convertToMB($$data_ref[1])),
	    year    => $year,
	    month   => $month,
	    day	    => $day,
	});
    }

    $sth->finish;
    
    $tp = HTML::Template->new (filename => './html/count/detailedDays.tmpl');
    $tp->param(	
		    fn	        => $query{fn},
		    year	=> $year,
		    month_word	=> $months[$month-1],
		    uid		=> $query{uid},
		    name	=> $UNIT->{$query{uid}}{name},
		    pid		=> $query{pid},
		    policy	=> $POLICY->{$query{pid}}{name},
		    flag	=> $UNIT->{$query{uid}}{policy}{$query{pid}},
		    stats_by_days  => \@stats_by_days,
		);
}

sub detailedHours
{
    my $dbh = &dbConnect;
    my @stats_by_hours;

    my $sth = $dbh->prepare("
	SELECT bytes_in, bytes_out, FROM_UNIXTIME(t_from, '%Y%m%d%H')
	FROM summary
	WHERE prefix='H'
	    AND unit_oid=".$dbh->quote(hex($query{uid}))."
	    AND policy_oid=".$dbh->quote(hex($query{pid}))."
	    AND FROM_UNIXTIME(t_from, '%Y%m%d')=".$dbh->quote($query{period})."
	ORDER by t_from desc");

    $sth->execute;
   
    my ($year, $month, $day);
    while (my $data_ref = $sth->fetchrow_arrayref)
    {
	my $hour;
	($year, $month, $day, $hour) = unpack('A4 A2 A2 A2', $$data_ref[2]);
   
	push(@stats_by_hours,
	{
	    mb_in   => sprintf("%.3f", &convertToMB($$data_ref[0])),
	    mb_out  => sprintf("%.3f", &convertToMB($$data_ref[1])),
	    hour    => $hour,
	});
    }

    $sth->finish;
    
    $tp = HTML::Template->new (filename => './html/count/detailedHours.tmpl');
    $tp->param(	
		    fn	        => $query{fn},
		    year	=> $year,
		    month	=> $month,
		    month_word	=> $months[$month-1],
		    day		=> $day,
		    uid		=> $query{uid},
		    name	=> $UNIT->{$query{uid}}{name},
		    pid		=> $query{pid},
		    policy	=> $POLICY->{$query{pid}}{name},
		    flag	=> $UNIT->{$query{uid}}{policy}{$query{pid}},
		    stats_by_hours  => \@stats_by_hours,
		);
}

1;
