## NeTAMS (www.netams.com) admintool by RamireZ (ramirez@ru.ru)
##
## UNITS MANAGEMENT MODULE

use strict;

# Load data
my ($UNIT, $UNIT_BYNAME)	= &unitLoadData;
my ($POLICY, $POLICY_BYNAME)	= &policyLoadData;
my ($QUOTA, $QUOTA_INFO)	= &quotaLoadData;
my $LOGIN			= &loginLoadData;

# Module for secrets file editing
my $secrets;
if ($cfg{secrets})
{
    require Edit::Secrets;

    $secrets = Edit::Secrets->new ($cfg{secrets_file});
    $secrets->read;
}

my %switch =
(
    default	=> \&showList,
    view	=> \&showEditUnit,
    setunit	=> \&makeEditUnit,
    addunit1	=> \&showAddUnit,
    addunit2	=> \&makeAddUnit,
    rm1		=> \&showDeleteUnit,
    rm2		=> \&makeDeleteUnit,
    addpol	=> \&addPol,
    rmpol	=> \&rmPol,
    setgrp	=> \&setGroup,
    mv		=> \&makeMove,
);
&{ &chooseAction(\%switch) };

########################
sub showList {
########################
    my @unitlist;
     
    foreach my $uid (keys %$UNIT) 
    {
	&_units_build_tree(\@unitlist, $uid, 0) if ($UNIT->{$uid}{parent} eq "<>"); 
    }
    
    $tp = HTML::Template->new(filename => './html/unit/showList.tmpl');
    $tp->param(
		    unitlist => \@unitlist,
		    grplist => &_createGrpList,
		);
}

########################
sub _units_build_tree {
########################

    # Arguments:
    # 0 - reference to unitlist array
    # 1 - uid
    # 2 - depth (number of parents)

    
    my ($unitlist, $uid, $depth) = @_;    

    my @childs_list;
    if ($UNIT->{$uid}{type} eq "group") 
    {
	@childs_list = split(/\ /, $UNIT->{$uid}{params});
    }
    
    push(@$unitlist,
	{
	    fn => $query{fn},
	    uid => $uid,
	    name => $UNIT->{$uid}{name},
	    parent => $UNIT->{$uid}{parent},
	    type => $UNIT->{$uid}{type},
	    isgroup => ($UNIT->{$uid}{type} eq "group" ? 1 : 0),
	    quotaon => $QUOTA->{$uid}{quota},
	    loginon => $LOGIN->{$uid}{status},
	    childsnum => scalar(@childs_list),
	    params => $UNIT->{$uid}{params},
	    indent => ($depth * 16),
	});

    # Sort childs list, in a way that units will be shown before groups
    @childs_list = sort
    {
	if ($UNIT_BYNAME->{$a}{type} eq "group" and $UNIT_BYNAME->{$b}{type} eq "group") { return 0 }
	elsif ($UNIT_BYNAME->{$a}{type} eq "group") { return 1 }
	else { return -1 }
    } @childs_list; 

    # If current unit is a group, execute units_build_tree for each child
    foreach (@childs_list) 
    {
	&_units_build_tree($unitlist, $UNIT_BYNAME->{$_}{oid}, $depth+1);
    }
}

########################
sub showEditUnit {
########################
    
    unless ($query{uid}) { &showList; return; }
    
    # Create two lists with all policies assigned to current unit:
    # first with account policies
    # second with firewall policies
    my (@acctpols, @fwpols);
    foreach my $pid (split(",", $UNIT->{$query{uid}}{policy}{list}))
    {
	my %pol;
	
	$pol{fn} = $query{fn};
	$pol{uid} = $query{uid};
	$pol{pid} = $pid;
	$pol{polname} = $POLICY->{$pid}{name};
	$pol{flag_invert} = ($UNIT->{$query{uid}}{policy}{$pid} eq "!"  ? 1 : 0);
	$pol{flag_pass}	  = ($UNIT->{$query{uid}}{policy}{$pid} eq "%"  ? 1 : 0);
	$pol{flag_both}   = ($UNIT->{$query{uid}}{policy}{$pid} eq "%!" ? 1 : 0);
			
	if	($POLICY->{$pid}{type} eq "acct") { push(@acctpols,  \%pol); }	
	elsif	($POLICY->{$pid}{type} eq "fw")	  { push(@fwpols,    \%pol); }
    }
    
    # Check what to show in html:
    # if unit is not a group, show ip
    # if it's also a user, show email
    # if it's not a group and not a user, and secrets enabled, show password 
    my ($show_password, $password, $show_ip, $show_email);
    if ($UNIT->{$query{uid}}{type} ne "group")
    {
	$show_ip = 1;
	
	if ($UNIT->{$query{uid}}{type} eq "user")
	{
	    $show_email = 1;
	}
	elsif ($cfg{secrets})
	{
	    $show_password = 1;
	    if (defined($secrets->{'data'}{$UNIT->{$query{uid}}{name}}))
	    {
		# Assign the same number of "*" to $password 
		# as the number of characters in password from secrets file
		$password = "*" x length($secrets->{'data'}{$UNIT->{$query{uid}}{name}}->[1]);
	    }
	    
	}
    }
	 
    my ($acctpols_cbo, $fwpols_cbo) = &_createPolsCbo;
    
    $tp = HTML::Template->new(filename => './html/unit/showEditUnit.tmpl');
    $tp->param(
		    uid	    => $query{uid},
		    name    => $UNIT->{$query{uid}}{name},
		    type    => $UNIT->{$query{uid}}{type},
		    
		    parent_uid	=> $UNIT_BYNAME->{$UNIT->{$query{uid}}{parent}}{oid},
		    parent_name => $UNIT->{$query{uid}}{parent},
		    
		    show_password   => $show_password,
		    password	    => $password,
		    show_ip	    => $show_ip,
		    ip		    => $UNIT->{$query{uid}}{params},
		    show_email	    => $show_email,
		    email	    => $UNIT->{$query{uid}}{email},

		    nolp => ($UNIT->{$query{uid}}{nlp} eq "+" ? "checked" : 0),

		    grplist => &_createGrpList,

		    acctpols	    => \@acctpols,
		    fwpols	    => \@fwpols,
		    acctpols_cbo    => $acctpols_cbo,
		    fwpols_cbo	    => $fwpols_cbo,
		);
}

########################
sub makeEditUnit {
########################
    my $debug;
    my $cmdstr;
    my $change_secrets;
     
    $query{name}=~ s/ +/_/g;
    
    # Add name to cmdstr if it is changed
    if ($query{name} ne "" && $UNIT->{$query{uid}}{name} ne $query{name}) 
    { 
	$cmdstr.="name $query{name} ";
	# Activate switch to change secrets file later
	$change_secrets = 1;
    }
    
    # Add ip to cmdstr if this is not group and ip changed
    if ($UNIT->{$query{uid}}{type} ne "group" && $query{prms} ne "" && $UNIT->{$query{uid}}{params} ne $query{prms})
    {
	$cmdstr.="ip $query{prms} ";
	# Activate switch to change secrets file later
	$change_secrets = 1;
    } 
    
    # Add email to cmdstr if this is unit user and email changed
    if ($UNIT->{$query{uid}}{type} eq "user" && $query{email} ne "" && $UNIT->{$query{uid}}{email} ne $query{email})
    {
	$cmdstr.="email $query{email} ";	
    }
    
    # Check if no local pass checkbox is active
    if ($query{nolp})	{ $cmdstr.="no-local-pass " }
    else		{ $cmdstr.="no no-local-pass " }

    # Execute commands
    $debug.=netams_cmd("service $cfg{nas_prc}");
    $debug.=netams_cmd("unit $UNIT->{$query{uid}}{type} oid $query{uid} $cmdstr");
    $debug.=netams_cmd("exit");
    $debug.=netams_cmd("save");

    # Apply changes to secrets file
    if ($UNIT->{$query{uid}}{type} ne "group" && 
	$UNIT->{$query{uid}}{type} ne "user" &&
	$cfg{secrets} && 
	($query{password} ne "" || $change_secrets))
    {
	my $password;
	if ($query{password} !~ /^\**$/) { $password = $query{password}; }
	else { $password = $secrets->{'data'}{$UNIT->{$query{uid}}{name}}->[1]; }
	
	# If name of unit has changed delete old record,
	# later we will create new one with different name.
	if ($UNIT->{$query{uid}}{name} ne $query{name})
	{
	    $secrets->del ($UNIT->{$query{uid}}{name});
	}
	
	# If our unit is defined in secrets hash - edit record (first arg is 0).
	if (defined($secrets->{'data'}{$UNIT->{$query{uid}}{name}}))
	{
	    $secrets->edit (0, "$query{name}", "*", "$password", "$query{prms}");
	}
	# If it is a new record, add it (first arg is 1).
	else
	{
	    $secrets->edit (1, "$query{name}", "*", "$password", "$query{prms}");
	}
	# Write secrets file
	$secrets->write;
    }
	
    $tp = HTML::Template->new(filename => './html/redirect.tmpl');
    $tp->param(params => "&act=view&uid=$query{uid}");
}

########################
sub showAddUnit {
########################
    my ($acctpols_cbo, $fwpols_cbo) = &_createPolsCbo;
    
    $tp = HTML::Template->new(filename => './html/unit/showAddUnit.tmpl');
    $tp->param(
		    show_password   => $cfg{secrets},
			
		    grplist	    => &_createGrpList,
		    acctpols_cbo    => $acctpols_cbo,
		);
}

########################
sub makeAddUnit {
########################
    my $debug;
    
    $query{name}=~ s/ +/_/g;
 
    if ($query{name} eq "")
    { 
	$tp = HTML::Template->new(filename => './html/redirect.tmpl');
	return;
    }
  
    my $cmdstr = "unit $query{type} name $query{name}"; 
    if ($query{type} eq "host" or $query{type} eq "cluster")
    {
	if ($query{params} ne "")
	{
	    $cmdstr .= " ip $query{params}";
		
	    # Add password entry to the secrets file.
	    if ($cfg{secrets} and $query{password} !~ /^\**$/)
	    {
		$secrets->edit (1, "$query{name}", "*", "$query{password}", "$query{params}");
		$secrets->write;
	    }
	}
	else
	{
	    $cmdstr = "";
	}
    }
    elsif ($query{type} eq "user" && $query{params} ne "")
    {	
	$cmdstr .= " email $query{params}";
    }
    
    $cmdstr .= " parent $query{parent}" if ($cmdstr);
	
    $debug.=netams_cmd("service $cfg{nas_prc}");
    $debug.=netams_cmd("$cmdstr");
    $debug.=netams_cmd("exit");
    $debug.=netams_cmd("save");
	
    # Set quota for created unit.
    if ($query{quota} ne "" && $query{per} ne "" && $query{inoutsum} ne "") 
    {
	$debug.=&quotaSet($query{name}, $query{per}, "$query{quota}", $query{inoutsum});
    }

    # Set acct-policy for created unit.
    if ($query{pid} ne "" && $query{type} ne "")
    {
	$debug.=&policySet($query{type}, $query{name}, $POLICY->{$query{pid}}{type}, $query{flag}, $POLICY->{$query{pid}}{name});
    }
	
    $tp = HTML::Template->new(filename => './html/redirect.tmpl');
}

########################
sub showDeleteUnit {
########################
    my (@dellist, $delstr);
    
    foreach my $key (keys %query)
    {
	if ($key ne "act" && $key ne "gid" && $key ne "fn") 
	{
	    push(@dellist,
		{
		    uid => $key,
		    name => $UNIT->{$key}{name},
		    type => $UNIT->{$key}{type},
		});

	    $delstr .= "&$key=on";
	}
    }
    
    $tp = HTML::Template->new(filename => './html/unit/showDeleteUnit.tmpl');
    $tp->param(
			dellist => \@dellist,
			delstr => $delstr,
		    );
}

########################
sub makeDeleteUnit {
########################
    my $debug = netams_cmd("service $cfg{nas_prc}");
    foreach my $key (keys %query)
    {
	if ($key ne "act" && $key ne "gid" && $key ne "fn") 
	{
	    $debug .= netams_cmd("no unit $UNIT->{$key}{type} name $UNIT->{$key}{name}");
	    if ($cfg{secrets})
	    { 
		$secrets->del ($UNIT->{$key}{name});
		$secrets->write;
	    }
	}
    }
    $debug.=netams_cmd("exit");
    $debug.=netams_cmd("save");
	
	
    $tp = HTML::Template->new(filename => './html/redirect.tmpl');
}

########################
sub addPol {
########################
    my $debug = &policySet($UNIT->{$query{uid}}{type}, $UNIT->{$query{uid}}{name}, $POLICY->{$query{pid}}{type}, $query{flag}, $POLICY->{$query{pid}}{name});
    #print "<pre>$debug</pre>";
    
    $tp = HTML::Template->new(filename => './html/redirect.tmpl');
    $tp->param(params => "&act=view&uid=$query{uid}");
}

########################
sub rmPol {
########################
    my $cmdstr = "unit $UNIT->{$query{uid}}{type} name $UNIT->{$query{uid}}{name} no $POLICY->{$query{pid}}{type}-policy $POLICY->{$query{pid}}{name}";
	
    my $debug.=netams_cmd("service $cfg{nas_prc}");
    $debug.=netams_cmd($cmdstr);
    $debug.=netams_cmd("exit");
    $debug.=netams_cmd("save");
    #print "<pre>$debug</pre>";
    
    $tp = HTML::Template->new(filename => './html/redirect.tmpl');
    $tp->param(params => "&act=view&uid=$query{uid}");
}

########################
sub setGroup {
########################
    my $cmdstr = "unit $UNIT->{$query{uid}}{type} name $UNIT->{$query{uid}}{name}";
    if ($query{gid} eq "")	{ $cmdstr .= " no parent" }
    else			{ $cmdstr .= " parent $query{gid}" }
    
    my $debug	.= netams_cmd("service $cfg{nas_prc}");
    $debug	.= netams_cmd("$cmdstr");
    $debug	.= netams_cmd("exit");
    $debug	.= netams_cmd("save");
    #print "<pre>$debug</pre>";
	
    $tp = HTML::Template->new(filename => './html/redirect.tmpl');
    $tp->param(params => "&act=view&uid=$query{uid}");
}

########################
sub makeMove  {
########################
    my $debug = netams_cmd("service $cfg{nas_prc}");
    
    foreach my $key (keys %query) 
    {
	if ($key ne "act" && $key ne "gid" && $key ne "fn") 
	{
	    my $cmdstr = "unit $UNIT->{$key}{type} name $UNIT->{$key}{name}";
	    
	    if ($query{gid} eq "")  { $cmdstr .= " no parent" }
	    else		    { $cmdstr .= " parent $query{gid}" }
	    
	    $debug .= netams_cmd("$cmdstr");
	}
    }
    
    $debug.=netams_cmd("exit");
    $debug.=netams_cmd("save");
    
    $tp = HTML::Template->new(filename => './html/redirect.tmpl');
}

##
#  Second ring subs:
##

# Create list of all available groups except root (<>)
# for use as a template param
sub _createGrpList
{
    my @grplist;
    
    foreach (keys %$UNIT)
    {
	push(@grplist, { group => $UNIT->{$_}{name} }) if ($UNIT->{$_}{type} eq "group");
    }

    return \@grplist;
}

# Create two lists with all available policies:
# first with account policies
# second with firewall policies
# for use in a select tag
sub _createPolsCbo
{
    my (@acctpols_cbo, @fwpols_cbo);
    
    foreach my $pid (keys %$POLICY)
    {
	my %pol;

	$pol{pid}	= $pid;
	$pol{polname}   = $POLICY->{$pid}{name};
	
	if    ($POLICY->{$pid}{type} eq "acct") { push(@acctpols_cbo, \%pol) }
	elsif ($POLICY->{$pid}{type} eq "fw")   { push(@fwpols_cbo,   \%pol) }
    }

    return (\@acctpols_cbo, \@fwpols_cbo);
}

1;
